/*
 * ********************************************************************************************************************
 *
 *                      :*$@@%$*:                         ;:                ;;    ;;
 *                    :@@%!  :!@@%:                       %!             ;%%@@%$ =@@@@@@@%;     @%@@@%%%%@@@@@
 *                   :@%;       :$=                       %%$$$%$$         ;$$  ;$@=   !@$
 *                   =@!                                  %!              @ $=;%   !@@@%:      !$$$$$$$$$$$$$$=
 *                   =@*                                  %!              @ $= % %@=   =%@!      %=
 *              *$%%! @@=        ;=$%%%$*:                %!              @ $= % =%%%%%%@$      *%:         =%
 *            %@@!:    !@@@%=$@@@@%!  :*@@$:              %!              @ $= % $*     ;@      @*          :%*
 *          ;@@!          ;!!!;:         ;@%:      =======@%========*     @ $$ % $%*****$@     :@$=*********=@$
 *          $@*   ;@@@%=!:                *@*
 *          =@$    ;;;!=%@@@@=!           =@!
 *           %@$:      =@%: :*@@@*       %@=                    Copyright (c) 2016-2019.  北京上格云技术有限公司
 *            ;%@@$=$@@%*       *@@@$=%@@%;
 *               ::;::             ::;::                                              All rights reserved.
 *
 * ********************************************************************************************************************
 */

import { SGraphyItem, SMouseEvent } from "@sybotan-web/graphy/lib";
import { SColor, SPainter, SPoint, SRect } from "@sybotan-web/draw/lib";
import { Space } from "../types/Space";
import { Opt } from "../types/Opt";

/**
 * 空间item
 *
 * @author  郝建龙
 */
export class SpaceItem extends SGraphyItem {
    /** 空间所有数据  */
    data: Space;
    /** 空间轮廓线坐标list  */
    private readonly pointArr: SPoint[] = [];
    /** X坐标最小值  */
    private minX = 0;
    /** X坐标最大值  */
    private maxX = 0;
    /** Y坐标最小值  */
    private minY = 0;
    /** Y坐标最大值  */
    private maxY = 0;
    /**
     * 构造函数
     *
     * @param parent    指向父对象
     * @param data      空间数据
     */
    constructor(parent: SGraphyItem | null, data: Space) {
        super(parent);
        this.data = data;
        if (this.data.OutLine.length) {
            this.pointArr = this.data.OutLine[0].map(t => {
                return new SPoint(t.X, t.Y);
            });
        }
        this.minX = this.pointArr[0].x;
        this.maxX = this.minX;
        this.minY = this.pointArr[0].y;
        this.maxY = this.minY;
        for (let i = 1; i < this.pointArr.length; i++) {
            if (this.pointArr[i].x < this.minX) {
                this.minX = this.pointArr[i].x;
            }
            if (this.pointArr[i].y < this.minY) {
                this.minY = this.pointArr[i].y;
            }
            if (this.pointArr[i].x > this.maxX) {
                this.maxX = this.pointArr[i].x;
            }
            if (this.pointArr[i].y > this.maxY) {
                this.maxY = this.pointArr[i].y;
            }
        }
    } // Constructor

    /**
     * Item对象边界区域
     *
     * @return SRect
     */
    boundingRect(): SRect {
        return new SRect(
            this.minX,
            this.minY,
            this.maxX - this.minX,
            this.maxY - this.minY
        );
    } // Function boundingRect()

    /**
     * 判断点是否在区域内
     *
     * @param x
     * @param y
     */
    contains(x: number, y: number): boolean {
        let nCross = 0,
            point = [x, y],
            APoints = this.data.OutLine[0],
            length = APoints.length,
            p1,
            p2,
            i,
            xinters;
        p1 = APoints[0];
        for (i = 1; i <= length; i++) {
            p2 = APoints[i % length];
            if (
                point[0] > Math.min(p1.X, p2.X) &&
                point[0] <= Math.max(p1.X, p2.X)
            ) {
                if (point[1] <= Math.max(p1.Y, p2.Y)) {
                    if (p1.X != p2.X) {
                        //计算位置信息
                        xinters =
                            ((point[0] - p1.X) * (p2.Y - p1.Y)) /
                                (p2.X - p1.X) +
                            p1.Y;
                        if (p1.Y == p2.Y || point[1] <= xinters) {
                            nCross++;
                        }
                    }
                }
            }
            p1 = p2;
        }
        return nCross % 2 === 1;
    } // Function contains()

    /**
     * Item绘制操作
     *
     * @param   painter       painter对象
     * @param   rect          绘制区域
     */
    onDraw(painter: SPainter, rect?: SRect): void {
        if (this.visible) {
            painter.pen.color = SColor.Transparent;
            painter.brush.color = Opt.spaceColor;
            painter.pen.lineWidth = 200;
            painter.drawPolygon(this.pointArr);
        }
    } // Function onDraw()
} // Class SpaceItem
